@@ -34,10 +34,8 @@ from w3lib.encoding import html_body_declared_encoding, http_content_type_encodi
 from .headers import use_scrapy_headers
 from .page import PageMethod
 
-
 __all__ = ["ScrapyPlaywrightDownloadHandler"]
 
-
 PlaywrightHandler = TypeVar("PlaywrightHandler", bound="ScrapyPlaywrightDownloadHandler")
 
 logger = logging.getLogger("scrapy-playwright")
@@ -173,8 +171,8 @@ class ScrapyPlaywrightDownloadHandler(HTTPDownloadHandler):
         if self.default_navigation_timeout is not None:
             page.set_default_navigation_timeout(self.default_navigation_timeout)
 
-        # page.on("close", self._make_close_page_callback(context_name))
-        # page.on("crash", self._make_close_page_callback(context_name))
+        page.on("close", self._make_close_page_callback(context_name))
+        page.on("crash", self._make_close_page_callback(context_name))
         page.on("request", _make_request_logger("default"))
         page.on("response", _make_response_logger("default"))
         page.on("request", self._increment_request_stats)
@@ -244,12 +242,15 @@ class ScrapyPlaywrightDownloadHandler(HTTPDownloadHandler):
             logger.exception(f"[{request.url}] Error while downloading request")
         else:
             logger.info(f"[{request.url}] Return result")
+            if not page.is_closed():
+                await page.close()
             return result
-        finally:
-            await page.close()
-            self.stats.inc_value("playwright/page_count/closed")
-            self.context_semaphores['default'].release()
-
+        # finally:
+        #     logger.info(f"[{request.url}] Finally")
+        #     if page is not None and not page.is_closed():
+        #         await page.close()
+        #     self.stats.inc_value("playwright/page_count/closed")
+        #     self.context_semaphores['default'].release()
 
     async def _download_request_with_page(self, request: Request, page: Page) -> Response:
         start_time = time()
@@ -344,7 +345,7 @@ class ScrapyPlaywrightDownloadHandler(HTTPDownloadHandler):
         return close_browser_context_callback
 
     def _make_request_handler(
-        self, method: str, scrapy_headers: Headers, body: Optional[bytes], encoding: str = "utf8"
+            self, method: str, scrapy_headers: Headers, body: Optional[bytes], encoding: str = "utf8"
     ) -> Callable:
         async def _request_handler(route: Route, playwright_request: PlaywrightRequest) -> None:
             """Override request headers, method and body."""
